# MCP Server for Salesforce Metadata - Technical Details

## Overview

This Python-based MCP (Model Context Protocol) Server provides a secure bridge between Claude AI and Salesforce, enabling Claude to perform metadata operations, execute SOQL queries, generate test data, and search Salesforce documentation.

## Security Improvements

The server now securely loads all credentials from environment variables via a `.env` file instead of requesting them in chat. This ensures:

- Salesforce credentials are never exposed to Claude
- Credentials are not stored in conversation history
- API keys remain secure and separate from the application code
- Improved compliance with security best practices

## Environment Configuration

Create a `.env` file in the same directory as the server with the following variables:

```
# Salesforce Credentials
SF_USERNAME=your_salesforce_username
SF_PASSWORD=your_salesforce_password
SF_SECURITY_TOKEN=your_salesforce_security_token

# Google API Credentials (for Search Documentation tool)
GOOGLE_API_KEY=your_google_api_key
GOOGLE_CSE_ID=your_google_custom_search_engine_id
```

## MCP Tools

The server provides the following tools:

### Connection Tools
- `connect_to_salesforce`: Automatically connects to Salesforce using environment variables
- `get_salesforce_connection_status`: Checks if the connection is active and valid

### Metadata Tools
- `describe_metadata_types`: Lists available metadata types in your org
- `list_metadata`: Retrieves components of a specific metadata type
- `describe_object`: Provides detailed schema information for an object
- `create_custom_field`: Creates a new custom field on an object
- `create_custom_object`: Creates a new custom object
- `create_data_model`: Creates a complete data model with relationships

### SOQL Tools
- `generate_soql_query`: Creates a SOQL query from natural language
- `execute_soql_query`: Runs a SOQL query and returns results

### Test Data Tools
- `generate_test_data`: Creates realistic test data for Salesforce objects

### Documentation Tool
- `search_salesforce_documentation`: Searches official Salesforce documentation

## Claude Desktop Integration

To use this MCP server with Claude desktop:

1. Start the server: `python secure_mcp_server.py`
2. Create a `claude_desktop_config.json` file pointing to your MCP server
3. Set the server URL to `http://localhost:8000` (default port)
4. Restart your Claude desktop client

## Technical Implementation

- Uses `simple-salesforce` library for Salesforce API operations
- Implements the Model Context Protocol for standardized tool definitions
- Handles both REST and SOAP API calls to Salesforce
- Provides detailed error handling and user-friendly messages
- Automatically manages session persistence between operations

## Dependencies

- Python 3.8+
- simple-salesforce
- requests
- python-dotenv
- fastmcp (MCP server implementation)
- google-api-python-client (optional, for documentation search)

## Error Handling

The server includes comprehensive error handling for:
- Missing environment variables
- Authentication failures
- API rate limiting
- Network connectivity issues
- Invalid metadata operations

## Performance Considerations

- Connection state is maintained between operations
- Schema information is cached to reduce API calls
- Bulk operations are used where appropriate for large data sets
